#include "ctrl.h"
#include "evilwm.h"
#include "str.h"
#include "parser.h"
#include "screen.h"

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

void ctrl_exec(char* cmd, ctrl_message_callback message_cb) {
    char* args[11];
    int argc = parse_cmd(cmd, message_cb, args);
    if (argc <= 0) {
        return;
    }

#ifdef STDIO
    {
        int i;
        LOG("Run command: args=%s", args[0]);
        for (i = 1; i < argc; i++) {
            LOG(", %s", args[i]);
        }
        LOG("\n");
    }
#endif
    if (!strcmp(args[0], "SHUTDOWN")) {
        if (argc != 1) {
            message_cb("SHUTDOWN takes no argument\n");
            return;
        }
        wm_running = 0;
    }
    else if (!strcmp(args[0], "VDESK")) {
        int vd;
        if (argc != 2) {
            message_cb("VDESK takes an argument\n");
            return;
        }
        if (!isdigit(args[1][0])) {
            message_cb("Vdesk must be a positive integer\n");
            return;
        }
        vd = atoi(args[1]);
        if (vd >= opt_vd) {
            message_cb("Too large vdesk number\n");
            return;
        }
        switch_vdesk(vd);
    }
    else if (!strcmp(args[0], "WIN")) {
        Client* client;
        const char* name;
        int index;
        int x, y, w, h, vd, focused;
        char mark;
        if (argc != 10) {
            message_cb("WIN takes 10 arguments\n");
            return;
        }
        name = args[1];
        if (!isdigit(args[2][0])) {
            message_cb("Window index must be a positive integer\n");
            return;
        }
        index = atoi(args[2]);
        if (!isdigit(args[3][0])) {
            message_cb("Window X must be a positive integer\n");
            return;
        }
        x = atoi(args[3]);
        if (!isdigit(args[4][0])) {
            message_cb("Window Y must be a positive integer\n");
            return;
        }
        y = atoi(args[4]);
        if (!isdigit(args[5][0])) {
            message_cb("Window W must be a positive integer\n");
            return;
        }
        w = atoi(args[5]);
        if (!isdigit(args[6][0])) {
            message_cb("Window H must be a positive integer\n");
            return;
        }
        h = atoi(args[6]);
        if (!isdigit(args[7][0]) && strcmp(args[7], "-1")) {
            message_cb("Window vdesk must be a positive integer or -1\n");
            return;
        }
        vd = atoi(args[7]);
        if (!isdigit(args[8][0])) {
            message_cb("Window focused must be a positive integer\n");
            return;
        }
        focused = atoi(args[8]);
        mark = args[9][0] ? args[9][1] ? atoi(args[9]) : args[9][0] : 0;

        for (client = head_client; client; client = client->next) {
            if (client->index == index && !strcmp(client->name, name)) {
                break;
            }
        }
        if (!client) {
            message_cb("No such window\n");
            return;
        }

        change_state(client, x, y, w, h, vd, focused, mark);
    }
    else {
        message_cb("Unknown command\n");
        return;
    }
    message_cb(NULL);
}

void ctrl_parse(char* cmds, int* parsed, ctrl_message_callback message_cb) {
    char* p;
    assert(cmds);
    assert(parsed);
    assert(*parsed >= 0);
    cmds += *parsed;

    for (p = cmds; *p; p++) {
        if (*p == '\n') {
            Str cmd = str_new(NULL);
            str_cat_buf(&cmd, cmds, p - cmds);
            *parsed += p - cmds + 1;
            ctrl_exec(cmd.buf, message_cb);
            cmds = p + 1;
            str_free(&cmd);
            LOG("parsed: %d\n", *parsed);
        }
    }
}
